local
function platforms_overlap(p1, p2)
    return p1.x1 < p2.x2 and p1.x2 > p2.x1 and p1.y1 < p2.y2 and p1.y2 > p2.y1
end

local
function platform_invades(p1, p2)
    local head_room = 100
    local w = p2.x2 - p2.x1
    local d = 50
    local proxy = {
        x1 = p2.x1 - d,
        x2 = p2.x2 + d,
        y1 = p2.y1,
        y2 = p2.y2 + head_room,
    }
    return platforms_overlap(proxy, p1)
end

local
function invades_any(candidate, platforms)
    for _, platform in ipairs(platforms) do
        if platform_invades(candidate, platform) or platform_invades(platform, candidate) then
            return true
        end
    end
    return false
end

local
function platform_reachable(from, to)
    local min_x_gap = 100
    local min_y_gap = 60
    local proxy = {
        x1 = from.x1 - min_x_gap,
        x2 = from.x2 + min_x_gap,
        y1 = from.y1 - min_y_gap,
        y2 = from.y2 + min_y_gap,
    }
    return platforms_overlap(proxy, to)
end

local
function any_reachable(candidate, platforms)
    if #platforms == 0 then
        return true
    end
    for _, platform in ipairs(platforms) do
        if platform_reachable(platform, candidate) then
            return true
        end
    end
    return false
end

local widths = {97, 160}
local img_names = {"platform2", "platform"}

function generate_platforms(orig_platforms, min_x, min_y, max_x, max_y)
    local num_platforms = 10

    local platforms
    repeat 
        platforms = table.clone(orig_platforms)

        for i = #platforms + 1, num_platforms do
            local platform
            local tries = 100
            repeat
                local p = math.random(#img_names)
                local w = widths[p]
                local h = 20
                local x = math.random() * (max_x - min_x) + min_x
                local y = math.random() * (max_y - min_y) + min_y
                local x1, y1, x2, y2 = x - w/2, y - h/2, x + w/2, y + h/2
                platform = {
                    x1 = x1,
                    y1 = y1,
                    x2 = x2,
                    y2 = y2,
                    type = p,
                    img_name = img_names[p]
                }
                tries = tries - 1
            until tries == 0 or (not invades_any(platform, platforms) and any_reachable(platform, platforms))
            if tries > 0 then
                table.insert(platforms, platform)
            end
        end
    until #platforms >= num_platforms / 2

    return platforms
end
